@@ -21,13 +21,10 @@
 
 import os
 import pickle
+import sys
 
 from nuitka import OutputDirectories
-from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
-    basestring,
-    to_byte,
-    xrange,
-)
+from nuitka.__past__ import basestring, to_byte, xrange
 from nuitka.Builtins import (
     builtin_anon_codes,
     builtin_anon_values,
@@ -38,6 +35,7 @@ from nuitka.Builtins import (
 from nuitka.codegen.Namify import namifyConstant
 from nuitka.containers.oset import OrderedSet
 from nuitka.PythonVersions import python_version
+from nuitka.utils.FileOperations import openTextFile
 
 
 class BuiltinAnonValue(object):
@@ -69,6 +67,8 @@ class BuiltinSpecialValue(object):
             return to_byte(0)
         elif self.value == "NotImplemented":
             return to_byte(1)
+        elif self.value == "Py_SysVersionInfo":
+            return to_byte(2)
         else:
             assert False, self.value
 
@@ -90,6 +90,8 @@ def _pickleAnonValues(pickler, value):
         pickler.save(BuiltinSpecialValue("Ellipsis"))
     elif value is NotImplemented:
         pickler.save(BuiltinSpecialValue("NotImplemented"))
+    elif value is sys.version_info:
+        pickler.save(BuiltinSpecialValue("Py_SysVersionInfo"))
     else:
         pickler.save_global(value)
 
@@ -101,7 +103,7 @@ class ConstantStreamWriter(object):
         self.count = 0
 
         filename = os.path.join(OutputDirectories.getSourceDirectoryPath(), filename)
-        self.file = open(filename, "wb")
+        self.file = openTextFile(filename, "wb")
         if python_version < 0x300:
             self.pickle = pickle.Pickler(self.file, -1)
         else:
@@ -113,6 +115,9 @@ class ConstantStreamWriter(object):
         self.pickle.dispatch[type(Ellipsis)] = _pickleAnonValues
         self.pickle.dispatch[type(NotImplemented)] = _pickleAnonValues
 
+        if type(sys.version_info) is not tuple:
+            self.pickle.dispatch[type(sys.version_info)] = _pickleAnonValues
+
     def addConstantValue(self, constant_value):
         self.pickle.dump(constant_value)
         self.count += 1
@@ -144,7 +149,7 @@ class ConstantAccessor(object):
     def getConstantCode(self, constant):
         # Use in user code, or for constants building code itself, many
         # constant types get special code immediately.
-        # pylint: disable=too-many-branches
+        # pylint: disable=too-many-branches,too-many-statements
         if constant is None:
             key = "Py_None"
         elif constant is True:
@@ -155,6 +160,8 @@ class ConstantAccessor(object):
             key = "Py_Ellipsis"
         elif constant is NotImplemented:
             key = "Py_NotImplemented"
+        elif constant is sys.version_info:
+            key = "Py_SysVersionInfo"
         elif type(constant) is type:
             # TODO: Maybe make this a mapping in nuitka.Builtins
 
