@@ -258,7 +258,7 @@ class ExpressionImportModuleFixed(ExpressionBase):
                 )
 
                 if module_filename is not None:
-                    package_module = self._consider(
+                    package_module = self._considerImport(
                         trace_collection=trace_collection,
                         module_filename=module_filename,
                         module_package=module_package,
