@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,7 +32,7 @@ import os
 from nuitka import Options
 
 from .ConstantRefNodes import makeConstantRefNode
-from .ExpressionBases import ExpressionBase
+from .ExpressionBases import ExpressionBase, ExpressionNoSideEffectsMixin
 
 
 class ExpressionModuleAttributeBase(ExpressionBase):
@@ -60,7 +60,10 @@ class ExpressionModuleAttributeBase(ExpressionBase):
     def getVariable(self):
         return self.variable
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
+        # These attributes can be expected to be present.
+
         return False
 
 
@@ -82,7 +85,7 @@ class ExpressionModuleAttributeFileRef(ExpressionModuleAttributeBase):
         if Options.getFileReferenceMode() != "runtime":
             result = makeConstantRefNode(
                 constant=self.variable.getModule().getRunTimeFilename(),
-                source_ref=self.getSourceReference(),
+                source_ref=self.source_ref,
             )
 
             return result, "new_expression", "Using original '__file__' value."
@@ -106,7 +109,7 @@ class ExpressionModuleAttributeNameRef(ExpressionModuleAttributeBase):
         if not Options.shallMakeModule():
             result = makeConstantRefNode(
                 constant=self.variable.getModule().getFullName().asString(),
-                source_ref=self.getSourceReference(),
+                source_ref=self.source_ref,
             )
 
             return result, "new_expression", "Using constant '__name__' value."
@@ -193,7 +196,7 @@ class ExpressionModuleAttributeSpecRef(ExpressionModuleAttributeBase):
         return self, None, None
 
 
-class ExpressionNuitkaLoaderCreation(ExpressionBase):
+class ExpressionNuitkaLoaderCreation(ExpressionNoSideEffectsMixin, ExpressionBase):
     __slots__ = ("provider",)
 
     kind = "EXPRESSION_NUITKA_LOADER_CREATION"
@@ -210,11 +213,3 @@ class ExpressionNuitkaLoaderCreation(ExpressionBase):
     def computeExpressionRaw(self, trace_collection):
         # Nothing can be done here.
         return self, None, None
-
-    def mayRaiseException(self, exception_type):
-        # Never raises an exception.
-        return False
-
-    def mayHaveSideEffects(self):
-        # No effect really by itself.
-        return False
