@@ -60,9 +60,57 @@ class NuitkaPluginResources(NuitkaPluginBase):
     def isAlwaysEnabled():
         return True
 
+    def _handleEasyInstallEntryScript(self, dist, group, name):
+        module_name = None
+        main_name = None
+
+        # First try metadata, which is what the runner also does first.
+        if self.metadata:
+            dist = self.metadata.distribution(dist.partition("==")[0])
+
+            for entry_point in dist.entry_points:
+                if entry_point.group == group and entry_point.name == name:
+                    module_name = entry_point.module
+                    main_name = entry_point.attr
+
+                    break
+
+        if module_name is None and self.pkg_resources:
+            entry_point = self.pkg_resources.get_entry_info(dist, group, name)
+
+            module_name = entry_point.module_name
+            main_name = entry_point.name
+
+        if module_name is None:
+            self.sysexit(
+                "Error, failed to resolve easy install entry script, is the installation broken?"
+            )
+
+        return r"""
+import sys, re
+sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
+import %(module_name)s
+sys.exit(%(module_name)s.%(main_name)s)
+""" % {
+            "module_name": module_name,
+            "main_name": main_name,
+        }
+
     def onModuleSourceCode(self, module_name, source_code):
         # Many cases to deal with, pylint: disable=too-many-branches
 
+        if module_name == "__main__":
+            match = re.search(
+                "\n# EASY-INSTALL-ENTRY-SCRIPT: '(.*?)','(.*?)','(.*?)'", source_code
+            )
+
+            if match is not None:
+                self.info(
+                    "Detected easy install entry script, compile time detecting entry point."
+                )
+
+                return self._handleEasyInstallEntryScript(*match.groups())
+
         # The importlib_resources backport has a problem with wanting source files
         # to exist, that won't be the case with standalone.
         if module_name == "importlib_resources._compat":
@@ -122,13 +170,32 @@ class NuitkaPluginResources(NuitkaPluginBase):
                     source_code = source_code.replace(match[0], "")
 
         if self.metadata:
-            for match in re.findall(
-                r"""\b((?:importlib_)?metadata\.version\(\s*['"](.*?)['"]\s*\))""",
+            for total, quote1, name, quote2 in re.findall(
+                r"""\b((?:importlib[_.])?metadata\.version\(\s*(['"]?)(.*?)(['"]?)\s*\))""",
                 source_code,
             ):
-                value = self.metadata.version(match[1])
-                value = repr(value)
+                if name == "__name__":
+                    name = module_name.asString()
+                    quote1 = quote2 = "'"
 
-                source_code = source_code.replace(match[0], value)
+                if quote1 == quote2:
+                    if quote1:
+                        try:
+                            value = self.metadata.version(name)
+                        except self.metadata.PackageNotFoundError:
+                            self.warning(
+                                "Cannot find requirement '%s' for '%s', expect potential run time problem."
+                                % (name, module_name)
+                            )
+
+                            continue
+                        except Exception:  # catch all, pylint: disable=broad-except
+                            self.sysexit(
+                                "Error, failed to resolve '%s', probably a plugin parsing bug for '%s' code."
+                                % (name, module_name)
+                            )
+                        else:
+                            value = repr(value)
+                            source_code = source_code.replace(total, value)
 
         return source_code
